import typing as t


class NoobError(Exception):
    '''Everybody Makes a Mistake'''

    pass


class TimeUnitError(NoobError):
    '''Typo Invalid and Empty'''

    pass


class DumbEmptyError(NoobError):
    pass


class CarelessFunctionError(NoobError):
    '''Ugh'''

    pass


class InvalidUrlError(NoobError):
    pass


def convert_time(time: t.Dict[int, str]) -> int:
    if not time:
        raise DumbEmptyError(
            'Put time value and unit in KV(dict) pairs , or just use seconds'
        )

    val, unit = list(time.items())[0]

    if unit == 'seconds':
        return val
    elif unit == 'minutes':
        return round(val / 60)
    elif unit == 'hours':
        return round(val / 360)
    else:
        raise TimeUnitError(
            f'{unit} is not a valid time unit, try to use "seconds minutes or hour..."'
        )


def callable_check(func: t.Callable):
    if not isinstance(func, t.Callable):
        raise CarelessFunctionError(f'{func} must be a function')


def drop(d: t.Dict, keys: t.List[str]):
    for k in keys:
        if k not in d.keys():
            # Test if there really exists the key need to be droped
            # if not, act like normal loop, and pretending nothing happend
            continue
        d.pop(k)
    return d



import os
import json
import httpx
import typing as t
import base64
from .. import utils
from ..base import ProcManager
from dao.model import OcrModel
from dao import YouDao as YouDaoClient

# TEST_TRANS_URL = "http://127.0.0.1:8000/search/"
# TEST_OCR_URL = "http://127.0.0.1:8000/ocr/"

TRANS_URL = "https://ke.study.163.com/code/textTrans"
OCR_URL = "https://ke.study.163.com/code/ocrTrans"

class ReqBuilder:
    @property
    def use_sdk(self):
        return self.appkey and self.appsecret

    @property
    def use_token(self):
        return bool(self.token)

    def __init__(self, serve_name: str) -> None:
        self.name = serve_name
        self.token = (
            os.environ.get("token")
            if os.environ.get("token")
            else os.environ.get("TOKEN")
        )  # wow ...
        self.appkey = os.environ.get("APP_KEY")
        self.appsecret = os.environ.get("APP_SECRET")
        self.payload = {"url": None, "headers": {}, "json": {}}
        self._post: t.Callable
        # self.req_setup(serve_name)

    def req_setup(self, name: str):
        if self.use_token:
            self.appkey = None
            self.appsecret = None
            servdict = {"trans": TRANS_URL, "ocr": OCR_URL}
            url = servdict[name]
            self.payload["url"] = url
            self.payload["headers"]["token"] = self.token
            self.payload["json"].update(OcrModel.config)
            # The data model that my model.cofig have, but XJ's don't
            self._post = httpx.post
        elif self.use_sdk:
            assert isinstance(self.appkey, str)
            assert isinstance(self.appsecret, str)
            client = YouDaoClient(self.appkey, self.appsecret)
            methdict = {"trans": client.translator().search, "ocr": client.ocr().upload}
            self._post = methdict[name]
        else:
            raise Exception(
                '''To use youdao API you need to get APP_KEY and APP_SECRET\n
                For more detail to visit -> https://ai.youdao.com/'''
            )

    def post(self, **kwargs) -> httpx.Response:
        self.req_setup(self.name)
        if self.use_sdk:
            self.payload = kwargs
        elif self.use_token:
            self.payload["json"].update(kwargs)
            self.payload["timeout"] = 10.0

        res = self._post(**self.payload)
        return res


trans_req = ReqBuilder("trans")
ocr_req = ReqBuilder("ocr")


def translate_post(q: str, **kwargs):
    result = {}
    res = trans_req.post(q=q)
    result["result"] = res
    result.update(kwargs)
    return result


def translate_filter(
    result: httpx.Response, full=False, **rest
) -> t.Union[t.Dict, t.List, None]:
    d = {}
    # test code or errorCode exist
    content = result.json()
    if content.get("code") == 200:
        content = json.loads(content["data"])
    # data goes like { code:200,data:{zhiyun...} }
    # TODO needs to figure out this resp...

    basic = content.get('basic')
    if basic:
        # if basic exists, q must be one word,
        basic = utils.drop(basic, keys=["us-speech", "uk-speech"])
        basic["webdict"] = content.get('webdict').get('url')
        d.update(basic)
        if not full:
            d = d.get("explains")

    else:
        # else many words
        # now needs code == 300 condition test ...
        d["oneline"] = content["translation"]
        return d["oneline"][0]

    return d


def pic2base64(path: str, **kwargs):
    with open(path, 'rb') as f:
        # q = base64.b64encode(f.read())
        return base64.b64encode(f.read()).decode('utf-8')
        # return q.decode('utf-8')


def ocr_post(b64: t.ByteString) -> httpx.Response:
    res = ocr_req.post(img=b64)
    return res


def ocr_ret(res: httpx.Response, **kwargs) -> t.Optional[t.List]:
    text_list = []
    content = res.json()
    if content.get("code") == 200:
        content = json.loads(content["data"])

    Result = content.get("Result")
    if Result:
        items = Result.get('regions')
        for item in items:
            for l in item['lines']:
                text_list.append(l['text'])

        return text_list


def translate(q: str, full: bool = False):
    p = ProcManager(translate_post, translate_filter)
    return p(dict(q=q, full=full))


def easy_ocr(path: str):
    p = ProcManager(pic2base64, ocr_post, ocr_ret)
    return p(dict(path=path))


def simple_translate(q):
    # no no no , what i want is not in the 'basic'
    return translate(q, full=False)


# SIGH -> I used to struggle here, until that I found it's XJ's problem
# self.payload["headers"]["Accept"] = "*/*"
# self.payload["headers"]["Accept-Encoding"] = "gzip, defate, br"
# self.payload["headers"]["Connection"] = "keep-alive"
# funny = {
#     "detectType": "10012",
#     "langType": "auto",
#     "imageType": "1",
#     "docType": "json",
# }
# self.payload["json"].update(funny)



import typing as t
from .base import Action, action
from .procs import weather, youdao, browser, memegen, fileoperation
from dataclasses import dataclass


class do(Action):
    '''
    Do, do my work\n
    Do my dirty work, scapegoat\n
    '''

    procs = []

    def __init__(self, *args) -> None:
        assert isinstance(self.procs, list)
        for f in args:
            self.procs.append(f)


@dataclass
class web_screenshot(Action):
    url: str
    procs = [browser.web_screenshot]


@dataclass
class get_now_temp(Action):
    city: str
    ascii_graphs: bool = True
    procs = [weather.get_weather, weather.ret_weather]
    # Should add something like this
    # procs = [weather.get_weather, _p(weather.ret_weather,{'ascii_graphs':True})]
    # procs = [weather.get_weather, _p(weather.ret_weather,ascii_graphs=True)]
    # The idea here is turn {"ascii_graphs":True} into self.ascii_graphs=True


@dataclass
class get_simple_temp(Action):
    city: str
    ascii_graphs: bool = False
    procs = [weather.get_weather, weather.ret_weather]


@dataclass
class translate(Action):
    q: str
    full: bool = True
    procs = [youdao.translate_post, youdao.translate_filter]


@dataclass
class simple_translate(Action):
    q: str
    procs = [youdao.simple_translate]


@dataclass
class easy_ocr(Action):
    path: str
    procs = [youdao.pic2base64, youdao.ocr_post, youdao.ocr_ret]


@dataclass
class meme_gen(Action):
    name: str
    up: t.Optional[str]
    down: str
    procs = [memegen.get_meme]


@dataclass
class list_all(Action):
    path: str
    procs = [fileoperation.list_all]


@dataclass
class create(Action):
    path: str
    procs = [fileoperation.create]


@dataclass
class create_folder(Action):
    path: str
    procs = [fileoperation.create_folder]


class find:
    def __init__(self, path: t.Text, pattern: t.Text, recursive=False) -> None:
        self.handler = fileoperation.walk if recursive else fileoperation.find
        self._list = self.handler(path=path, pattern=pattern)
        if isinstance(self._list, t.Generator):
            self._list = list(self._list)
            # when self.__dict__ deepcopy it goes nuts
            # Maybe I can do self.pipe(seed) without deepcopy...

    @action([fileoperation._delete_all])
    def delete(self) -> Action:
        return dict(f_list=self._list)

    @action([fileoperation._rename_all])
    def rename(self, with_suffix: str = None, with_preafix: str = None) -> Action:
        return locals()

    def zip(self):
        ...

    @action([fileoperation])
    def unzip(self):
        ...

    @action([fileoperation._move_all])
    def move(self, dst: str) -> Action:
        return dict(f_list=self._list, dst=dst)



from path import Path
from fs.copy import copy_fs
from fs.zipfs import ZipFS
from fs.osfs import OSFS
import typing as t


# Tested it works
def walk(path: str, pattern: str):
    return Path(path).walk(pattern)


# Tested it works
def list_all(path: str):
    # clean up.
    return Path(path).listdir()


# Tested ,It works
def find(path: str, pattern: str, **kwd):
    return Path(path).listdir(pattern)


# Tested it works
def create(path: str, **kwd):
    # Considering when dir not exist
    return Path(path).touch()


# Tested it works(Almost perfect)
def smart_delete(path: str):
    p = Path(path)
    if p.isdir():
        p.rmtree()
    elif p.isfile():
        p.remove()


# Tested it works(But feel... a litte weird?)
def delete(path: str, **kwd):
    p = Path(path)
    if p.isdir():
        # must be an exist dir
        print(
            f"The '{p}' is a dir(folder), and will not be deleted, if you want to"
            " delete a folder try to use delete_folder"
        )
        return
    p.remove()


def rmdir(path: str):
    p = Path(path)
    p.rmtree_p()


def rmdir_all(f_list: t.List[Path]):
    for p in f_list:
        rmdir(p)


# Tested ,it works
def _delete_all(f_list: list):
    for f in f_list:
        smart_delete(f)


# Tested ,it works(first appear in lesson 13)
def zip(src: str, dst: str):
    src, dst = Path(src), Path(dst)
    ffs = copy_fs(OSFS(dst), ZipFS(src))

    return ffs


def _zip_all(f_list: t.List[Path]):
    for f in f_list:
        ...


def unzip(src: str, dst: str):
    src, dst = Path(src), Path(dst)
    ffs = copy_fs(ZipFS(src), OSFS(dst, create=True))
    return ffs


def _upzip_all(f_list: t.List[Path]):
    for f in f_list:
        unzip(f, f.stem)


# It works(Not done yet)
def rename(
    path: str, new_name: str = None, with_suffix: str = None, with_prefix: str = None
):
    if new_name:
        if with_suffix or with_prefix:
            raise ValueError(
                "You don't need to do that, considering this"
                " way: 'prefix-newname.suffix'"
            )
        suffix = Path(path).ext
        _new_name = Path(path).dirname() / new_name + suffix
    elif with_suffix and with_prefix:
        # if new_name exists... wow...
        _new_name = Path(path).dirname() / with_prefix + Path(path).stem + with_suffix
    elif with_suffix:
        if new_name:
            raise ValueError("You can just pass 'newname.suffix' to new_name ")
        _new_name = Path(path).with_suffix(with_suffix)
    elif with_prefix:
        if new_name:
            raise ValueError("You can just pass 'prefix-newname.suffix' to new_name ")
        _new_name = Path(path).dirname() / with_prefix + Path(path).name

    else:
        raise ValueError("Need to pass at least one argument")

    Path(path).rename(_new_name)


def _rename_all(f_list: t.List[Path], with_prefix: str = None, with_suffix: str = None):

    if not with_prefix and not with_prefix:
        raise ValueError("To change files name, Must have a prefix or suffix ")

    for f in f_list:
        rename(f, with_suffix=with_suffix, with_prefix=with_prefix)


def create_folder(path: str, **kwd):
    p = Path(path)
    p.mkdir()


# Fixed, then test it.
def move(src: str, dst: str):
    p = Path(src)
    p.move(dst)


def _move_all(f_list: t.List[Path], dst: str):
    for f in f_list:
        f.move(dst)



import httpx
from pyquery import PyQuery
from ..base import ProcManager


def get_weather(city, ascii_graphs: bool = True, lang: str = "en", **kwargs) -> dict:
    res = httpx.get(
        f'http://wttr.in/{city}?lang={lang}{ "&T0" if ascii_graphs else "&format=j1"}'
    )
    return dict(res=res, ascii_graphs=ascii_graphs)


def ret_weather(res: httpx.Response, ascii_graphs: bool, **kwargs) -> str:
    if ascii_graphs:
        pq = PyQuery(res.text)
        result = pq.find('pre')
        return str(result.text(squash_space=False))
    temp = res.json().get('current_condition')
    return temp[0].get('FeelsLikeC')


def get_now_temp(city: str, ascii_graphs: bool = True):
    p = ProcManager(get_weather, ret_weather)
    return p(dict(city=city, ascii_graphs=ascii_graphs))


def get_simple_temp(city: str):
    return get_now_temp(city, ascii_graphs=False)



import httpx
import typing as t

url = "https://api.memegen.link/images/"


meme_list = [
    'aag',
    'ackbar',
    'afraid',
    'agnes',
    'aint-got-time',
    'ams',
    'ants',
    'apcr',
    'atis',
    'away',
    'awesome',
    'awesome-awkward',
    'awkward',
    'awkward-awesome',
    'bad',
    'badchoice',
    'bd',
    'bender',
    'bihw',
    'biw',
    'blb',
    'boat',
    'both',
    'bs',
    'buzz',
    'captain',
    'captain-america',
    'cb',
    'cbg',
    'center',
    'ch',
    'cheems',
    'chosen',
    'cmm',
    'crazypills',
    'cryingfloor',
    'db',
    'dg',
    'disastergirl',
    'dodgson',
    'doge',
    'dragon',
    'drake',
    'ds',
    'dsm',
    'dwight',
    'elf',
    'ermg',
    'fa',
    'facepalm',
    'fbf',
    'feelsgood',
    'fetch',
    'fine',
    'firsttry',
    'fmr',
    'fry',
    'fwp',
    'gandalf',
    'gb',
    'gears',
    'ggg',
    'gru',
    'grumpycat',
    'hagrid',
    'happening',
    'harold',
    'hipster',
    'home',
    'icanhas',
    'imsorry',
    'inigo',
    'interesting',
    'ive',
    'iw',
    'jd',
    'jetpack',
    'joker',
    'jw',
    'keanu',
    'kermit',
    'kk',
    'kombucha',
    'leo',
    'live',
    'll',
    'lrv',
    'mb',
    'michael-scott',
    'millers',
    'mini-keanu',
    'mmm',
    'money',
    'mordor',
    'morpheus',
    'mouth',
    'mw',
    'nice',
    'noidea',
    'ntot',
    'oag',
    'officespace',
    'older',
    'oprah',
    'patrick',
    'persian',
    'philosoraptor',
    'pigeon',
    'ptj',
    'puffin',
    'red',
    'regret',
    'remembers',
    'reveal',
    'right',
    'rollsafe',
    'sad-biden',
    'sad-boehner',
    'sad-bush',
    'sad-clinton',
    'sad-obama',
    'sadfrog',
    'saltbae',
    'same',
    'sarcasticbear',
    'sb',
    'scc',
    'sf',
    'sk',
    'ski',
    'snek',
    'soa',
    'sohappy',
    'sohot',
    'soup-nazi',
    'sparta',
    'spiderman',
    'spongebob',
    'ss',
    'stew',
    'stonks',
    'stop-it',
    'success',
    'tenguy',
    'toohigh',
    'tried',
    'trump',
    'ugandanknuck',
    'whatyear',
    'winter',
    'wkh',
    'wonka',
    'worst',
    'xy',
    'yallgot',
    'yodawg',
    'yuno',
    'zero-wing',
]


def get_meme(name: str, down: str, up: t.Optional[str] = None):

    if name not in meme_list:
        raise Exception(f"Sorry we don't have meme pic named {name} yet.")

    base = url + name
    full: str
    caps = []
    for w in [up, down]:
        if not w:
            continue
        _ = (
            w.replace(" ", "_")
            .replace("?", "~q")
            .replace("&", "~h")
            .replace("#", "~a")
            .replace("'", '"')
        )
        caps.append(_)

    if len(caps) == 2:
        up, down = caps
        full = f"{base}/{up}/{down}.jpg"
    elif len(caps) == 1:
        down = caps[0]
        full = f"{base}/_/{down}.jpg"
    else:
        raise Exception("One meme goes with two lines: Up and Down")

    with httpx.stream("GET", full) as r:
        with open(f"{name}.jpg", 'wb+') as f:
            for chunk in r.iter_bytes():
                f.write(chunk)
